/**
 * DEAgent - an agent in a Discrete Event simulation
 * A DEAgent doesn't act().
 * A DEAgent adds events to the queue, and requests resources.
 **/
public class DEAgent extends Agent {
  ///// Fields
    
  /** For support of resources */
  public boolean blocked;
  
  public void init(Simulation thisSim){
    super.init(thisSim);
    
    // Start out unblocked
    blocked = false;
  }
  
  /// Methods ///
  /** Are we blocked? */
  public boolean isBlocked(){return blocked;}
  
  /** Are we ready to be unlocked? */
  public boolean isReady(Resource res) {
    //As a default, we're ready!
    return true;}
  
  /** Provide a time that's not negative */
  public double validTime(double delay){
    if (delay < 0) 
      // Don't allow a negative delay
    {return ((DESimulation) simulation).getTime();}
    else {return (delay + ((DESimulation) simulation).getTime());}
  } // validTime
  
  /**
   * Block myself for the input resource
   * @param resource the desired resource
   **/
  public void waitFor(Resource resource){
    // I'm now blocked
    blocked = true;
    // Put me on the waiting list
    resource.addToList(this);
  }

  /**
   * I've been unblocked!
   * @param resource the desired resource
   **/
  public void unblocked(Resource resource){
    // I'm now blocked
    blocked = false;
    // Assume the subclass will do something useful with this.
  }

  /**
   * Process an event.
   * Default is to do nothing with it.
   **/
  public void processEvent(int message){}
  
  ////////////////////////////// Constructors ////////////////////////
  // Copy this section AS-IS into subclasses, but rename Agent to 
  // Your class.
  
  /**
   * Constructor that takes the model display (the original
   * position will be randomly assigned)
   * @param modelDisplayer thing that displays the model
   * @param thisSim my simulation
   */
  public DEAgent (ModelDisplay modelDisplayer,Simulation thisSim)
  {
    super(randNumGen.nextInt(modelDisplayer.getWidth()),
          randNumGen.nextInt(modelDisplayer.getHeight()),
          modelDisplayer, thisSim);
  }
  
  /** Constructor that takes the x and y and a model
   * display to draw it on
   * @param x the starting x position
   * @param y the starting y position
   * @param modelDisplayer the thing that displays the model
   * @param thisSim my simulation
   */
  public DEAgent (int x, int y, ModelDisplay modelDisplayer, 
               Simulation thisSim) 
  {
    // let the parent constructor handle it
    super(x,y,modelDisplayer,thisSim);
  }
}
