import java.awt.Color;
import java.util.LinkedList;

/**
 * WolfAgent -- Wolf as a subclass of Agent
 **/
public class WolfAgent extends Agent {
  /** class constant for how far wolf can smell */
  private static final double SMELL_RANGE = 50;

    /** class constant for how close before wolf can attack */
  private static final double ATTACK_RANGE = 30;
  
  /** Collection of all Wolves */
  public static LinkedList allWolves = new LinkedList();
  
  // My position
  public int myX;
  public int myY;
  
  /**
   * Initialize, by adding to Wolf list
   **/
  public void init(Simulation thisSim){
    // Do the normal initializations
    super.init(thisSim);
    
    // Make it brown
    setColor(Color.gray);
    
    // Add to list of Wolves
    allWolves.add(this);
    
    myX = this.getXPos();
    myY = this.getYPos();
  }
  
 /**
   * Chase and eat the deer
   **/
    /**
   * Method to act during a time step 
   * pick a random direction and move some random amount up to top speed
   */
  public void act()
  {
    // get the closest deer within smelling range
    DeerAgent closeDeer = (DeerAgent) getClosest(SMELL_RANGE,
                                DeerAgent.allDeer);
    if (closeDeer != null)
    {
      // Turn torward deer
      this.turnToFace(closeDeer);
      // How much to move?  How about minimum of maxSpeed 
      // or distance to deer?
      this.forward((int) Math.min(speed,
                   closeDeer.getDistance(this.getXPos(),this.getYPos())));
    }

    // get the closest deer within the attack distance
    closeDeer = (DeerAgent) getClosest(ATTACK_RANGE,
                           DeerAgent.allDeer);
    
    if (closeDeer != null)
    {
      this.moveTo(closeDeer.getXPos(),
                  closeDeer.getYPos());
      closeDeer.die();
    }
    
    else // Otherwise, wander aimlessly
    {
      
      super.act();
    } // end else
  } // end act()


    ////////////////////////////// Constructors ////////////////////////
  // Copy this section AS-IS into subclasses, but rename Agent to 
  // Your class.
  
  /**
   * Constructor that takes the model display (the original
   * position will be randomly assigned)
   * @param modelDisplayer thing that displays the model
   * @param thisSim my simulation
   */
  public WolfAgent (ModelDisplay modelDisplayer,Simulation thisSim)
  {
    super(randNumGen.nextInt(modelDisplayer.getWidth()),
          randNumGen.nextInt(modelDisplayer.getHeight()),
          modelDisplayer,thisSim);
  }
  
  /** Constructor that takes the x and y and a model
   * display to draw it on
   * @param x the starting x position
   * @param y the starting y position
   * @param modelDisplayer the thing that displays the model
   * @param thisSim my simulation
   */
  public WolfAgent (int x, int y, ModelDisplay modelDisplayer, 
               Simulation thisSim) 
  {
    // let the parent constructor handle it
    super(x,y,modelDisplayer,thisSim);
  }
  

}
